import peasy.*;
import processing.opengl.*;

PeasyCam cam;

int totalSegments = 10;

PVector[] orbit = new PVector[totalSegments];
Segment[] segments = new Segment[totalSegments];

PVector segmentOffset;

float margins = .1;
float rootScalar = 100;
float rootRadius = 230;
float rootNoiseRadius = 100;
float rootNoiseSpeed = .0125;

float rootHighlightSpeed = .5;
float rootSpinSpeedRange = .1;
float rootSpinSpeed = .1;

float elastictySmoothness = .7;
float elastictyRigidity = .2;
float elastictySpread = 3;

void setup() {
	size(1200,800,OPENGL); 
  
    cam = new PeasyCam(this, 1000);
    cam.setMinimumDistance(50);
    cam.setMaximumDistance(2000);
    cam.lookAt(0, 0, 0, 110);
    cam.setActive(true);  // false to make this camera stop responding to mouse
    
    // --------------------------------------------------------------------
    // parse data
    // --------------------------------------------------------------------
    createControls();

    // --------------------------------------------------------------------
    // parse data
    // --------------------------------------------------------------------

	for(int i=0;i<totalSegments;i++){
		segments[i] = new Segment();
		if(i==0){
			segments[i].previousSegment = segments[0];
		} else {
			segments[i].previousSegment = segments[i-1];
		}
		segments[i].seed = i;
		segments[i].normalizedSeed = float(i+1)/float(totalSegments);
		segments[i].randomizeDirection();
		segments[i].createRelatives();
		segments[i].resetRelatives();
		segments[i].createAbsolutes();
	}

}

void draw() {
	background(15);

	cam.beginHUD();
	if(isAxis){
		drawScope();
	}
	drawControls();
	cam.endHUD();
	
	setUpLighting();
	parseContent();
	drawContent();
}

void parseContent(){
	int i;

	segmentOffset = new PVector(0,0,0);
	
	for(i=0;i<totalSegments;i++){
		segments[i].setControlParams();
		segments[i].createRelatives();
		segments[i].resetRelatives();
		calculateSegmentPosition(i);
	}
	
	// first
	i = 0;
	segments[i].setPreviousOrbit(segments[i].startOrbit);
	segments[i].setNextOrbit(segments[i+1].startOrbit);
	segments[i].parseLerps();
	// middle
	for(i=1;i<(totalSegments-1);i++){
		segments[i].setPreviousOrbit(segments[i-1].endOrbit);
		segments[i].setNextOrbit(segments[i+1].startOrbit);
		segments[i].parseLerps();
	}
	// last
	i = totalSegments-1;
	segments[i].setPreviousOrbit(segments[i-1].endOrbit);
	segments[i].setNextOrbit(segments[i].endOrbit);
	segments[i].parseLerps();
}

void drawContent() {
	int i;
	if(isShape){
		for(i=0;i<totalSegments;i++){
			segments[i].drawShapeLines();
		}
	}
	if(isHighlight){
		for(i=0;i<totalSegments;i++){
			segments[i].drawHighlights();
		}
	}
	if(isMeshFill){
		for(i=0;i<totalSegments;i++){
			segments[i].drawMeshFill();
		}
	}
	if(isMeshWire){
		for(i=0;i<totalSegments;i++){
			segments[i].drawMeshWire();
		}
	}
	if(isWire){
		for(i=0;i<totalSegments;i++){
			segments[i].drawWireframes();
		}
	}
	if(isGuide){
		for(i=0;i<totalSegments;i++){
			segments[i].drawGuides();
		}
	}
	if(isAxis){
		drawAxes(400);
	}
}


void calculateSegmentPosition(int i){
	segments[i].createAbsolutes();
	segments[i].updateAbsolutes(segmentOffset);
	segments[i].parse();     
	segmentOffset.add(segments[i].endRelativeVec.get());
}

void keyPressed() {
	if (key == 'r' || key == 'R') {
		reset();
	}
}

void reset(){
	for(int i=0;i<totalSegments;i++){
		segments[i].randomizeDirection();
		segments[i].createRelatives();
		segments[i].setControlParams();
		segments[i].resetRelatives();
		segments[i].createAbsolutes();
	}
}

void setUpLighting(){
	// directionalLight(255, 255, 255, 0, 0, -1);
	ambientLight(25, 25, 25);
	// lights();
	spotLight(	255, 255, 255, 
			0, 0, 1000, 
			0, 0, -1, 
			PI/2, 1); 
}


void resetButton(int theValue) {
	reset();
}
void controlEvent(ControlEvent theEvent) {
    cam.setActive(false);  // false to make this camera stop responding to mouse
}

void mouseReleased() {
    cam.setActive(true);  // false to make this camera stop responding to mouse
}
